@@ -3,8 +3,12 @@ use std::net::{IpAddr, Ipv4Addr, Ipv6Addr, SocketAddr, SocketAddrV4, SocketAddrV
 use crate::de::{Decode, Decoder, PackDecoder, VariantDecoder};
 use crate::en::{Encode, Encoder, SequenceEncoder, VariantEncoder};
 use crate::error::Error;
+use crate::mode::Mode;
 
-impl<Mode> Encode<Mode> for Ipv4Addr {
+impl<M> Encode<M> for Ipv4Addr
+where
+    M: Mode,
+{
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
@@ -14,7 +18,10 @@ impl<Mode> Encode<Mode> for Ipv4Addr {
     }
 }
 
-impl<'de, Mode> Decode<'de, Mode> for Ipv4Addr {
+impl<'de, M> Decode<'de, M> for Ipv4Addr
+where
+    M: Mode,
+{
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
@@ -24,7 +31,10 @@ impl<'de, Mode> Decode<'de, Mode> for Ipv4Addr {
     }
 }
 
-impl<Mode> Encode<Mode> for Ipv6Addr {
+impl<M> Encode<M> for Ipv6Addr
+where
+    M: Mode,
+{
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
@@ -34,7 +44,10 @@ impl<Mode> Encode<Mode> for Ipv6Addr {
     }
 }
 
-impl<'de, Mode> Decode<'de, Mode> for Ipv6Addr {
+impl<'de, M> Decode<'de, M> for Ipv6Addr
+where
+    M: Mode,
+{
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
@@ -44,7 +57,10 @@ impl<'de, Mode> Decode<'de, Mode> for Ipv6Addr {
     }
 }
 
-impl<Mode> Encode<Mode> for IpAddr {
+impl<M> Encode<M> for IpAddr
+where
+    M: Mode,
+{
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
@@ -53,13 +69,16 @@ impl<Mode> Encode<Mode> for IpAddr {
         let variant = encoder.encode_variant()?;
 
         match self {
-            IpAddr::V4(v4) => variant.insert::<Mode, _, _>(0usize, v4),
-            IpAddr::V6(v6) => variant.insert::<Mode, _, _>(0usize, v6),
+            IpAddr::V4(v4) => variant.insert::<M, _, _>(0usize, v4),
+            IpAddr::V6(v6) => variant.insert::<M, _, _>(0usize, v6),
         }
     }
 }
 
-impl<'de, Mode> Decode<'de, Mode> for IpAddr {
+impl<'de, M> Decode<'de, M> for IpAddr
+where
+    M: Mode,
+{
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
@@ -67,16 +86,16 @@ impl<'de, Mode> Decode<'de, Mode> for IpAddr {
     {
         let mut variant = decoder.decode_variant()?;
 
-        let this = match variant.tag().and_then(<usize as Decode<Mode>>::decode)? {
+        let this = match variant.tag().and_then(<usize as Decode<M>>::decode)? {
             0 => Self::V4(
                 variant
                     .variant()
-                    .and_then(<Ipv4Addr as Decode<Mode>>::decode)?,
+                    .and_then(<Ipv4Addr as Decode<M>>::decode)?,
             ),
             1 => Self::V6(
                 variant
                     .variant()
-                    .and_then(<Ipv6Addr as Decode<Mode>>::decode)?,
+                    .and_then(<Ipv6Addr as Decode<M>>::decode)?,
             ),
             index => {
                 return Err(<D::Error as Error>::invalid_variant_tag("IpAddr", index));
@@ -88,63 +107,78 @@ impl<'de, Mode> Decode<'de, Mode> for IpAddr {
     }
 }
 
-impl<Mode> Encode<Mode> for SocketAddrV4 {
+impl<M> Encode<M> for SocketAddrV4
+where
+    M: Mode,
+{
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
         let mut pack = encoder.encode_pack()?;
-        pack.push::<Mode, _>(self.ip())?;
-        pack.push::<Mode, _>(self.port())?;
+        pack.push::<M, _>(self.ip())?;
+        pack.push::<M, _>(self.port())?;
         pack.end()
     }
 }
 
-impl<'de, Mode> Decode<'de, Mode> for SocketAddrV4 {
+impl<'de, M> Decode<'de, M> for SocketAddrV4
+where
+    M: Mode,
+{
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
         D: Decoder<'de>,
     {
         let mut unpack = decoder.decode_pack()?;
-        let ip = unpack.next().and_then(<Ipv4Addr as Decode<Mode>>::decode)?;
-        let port = unpack.next().and_then(<u16 as Decode<Mode>>::decode)?;
+        let ip = unpack.next().and_then(<Ipv4Addr as Decode<M>>::decode)?;
+        let port = unpack.next().and_then(<u16 as Decode<M>>::decode)?;
         Ok(SocketAddrV4::new(ip, port))
     }
 }
 
-impl<Mode> Encode<Mode> for SocketAddrV6 {
+impl<M> Encode<M> for SocketAddrV6
+where
+    M: Mode,
+{
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
         let mut pack = encoder.encode_pack()?;
-        pack.push::<Mode, _>(self.ip())?;
-        pack.push::<Mode, _>(self.port())?;
-        pack.push::<Mode, _>(self.flowinfo())?;
-        pack.push::<Mode, _>(self.scope_id())?;
+        pack.push::<M, _>(self.ip())?;
+        pack.push::<M, _>(self.port())?;
+        pack.push::<M, _>(self.flowinfo())?;
+        pack.push::<M, _>(self.scope_id())?;
         pack.end()
     }
 }
 
-impl<'de, Mode> Decode<'de, Mode> for SocketAddrV6 {
+impl<'de, M> Decode<'de, M> for SocketAddrV6
+where
+    M: Mode,
+{
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
         D: Decoder<'de>,
     {
         let mut unpack = decoder.decode_pack()?;
-        let ip = unpack.next().and_then(<Ipv6Addr as Decode<Mode>>::decode)?;
-        let port = unpack.next().and_then(<u16 as Decode<Mode>>::decode)?;
-        let flowinfo = unpack.next().and_then(<u32 as Decode<Mode>>::decode)?;
-        let scope_id = unpack.next().and_then(<u32 as Decode<Mode>>::decode)?;
+        let ip = unpack.next().and_then(<Ipv6Addr as Decode<M>>::decode)?;
+        let port = unpack.next().and_then(<u16 as Decode<M>>::decode)?;
+        let flowinfo = unpack.next().and_then(<u32 as Decode<M>>::decode)?;
+        let scope_id = unpack.next().and_then(<u32 as Decode<M>>::decode)?;
         Ok(Self::new(ip, port, flowinfo, scope_id))
     }
 }
 
-impl<Mode> Encode<Mode> for SocketAddr {
+impl<M> Encode<M> for SocketAddr
+where
+    M: Mode,
+{
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
@@ -153,13 +187,16 @@ impl<Mode> Encode<Mode> for SocketAddr {
         let variant = encoder.encode_variant()?;
 
         match self {
-            SocketAddr::V4(v4) => variant.insert::<Mode, _, _>(0usize, v4),
-            SocketAddr::V6(v6) => variant.insert::<Mode, _, _>(1usize, v6),
+            SocketAddr::V4(v4) => variant.insert::<M, _, _>(0usize, v4),
+            SocketAddr::V6(v6) => variant.insert::<M, _, _>(1usize, v6),
         }
     }
 }
 
-impl<'de, Mode> Decode<'de, Mode> for SocketAddr {
+impl<'de, M> Decode<'de, M> for SocketAddr
+where
+    M: Mode,
+{
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
@@ -167,16 +204,16 @@ impl<'de, Mode> Decode<'de, Mode> for SocketAddr {
     {
         let mut variant = decoder.decode_variant()?;
 
-        let this = match variant.tag().and_then(<usize as Decode<Mode>>::decode)? {
+        let this = match variant.tag().and_then(<usize as Decode<M>>::decode)? {
             0 => Self::V4(
                 variant
                     .variant()
-                    .and_then(<SocketAddrV4 as Decode<Mode>>::decode)?,
+                    .and_then(<SocketAddrV4 as Decode<M>>::decode)?,
             ),
             1 => Self::V6(
                 variant
                     .variant()
-                    .and_then(<SocketAddrV6 as Decode<Mode>>::decode)?,
+                    .and_then(<SocketAddrV6 as Decode<M>>::decode)?,
             ),
             index => {
                 return Err(<D::Error as Error>::invalid_variant_tag(
